/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   Jack Transport Audit Utils                                            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version                                                               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* t_sink_audio.cpp
 *
 * JACK Transport client that listens for about 262144 samples, and reports
 * the frames where it observed a full-scale sample.
 */

#include <cstring>
#include <unistd.h>
#include <iostream>
#include <jack/jack.h>
#include <jack/transport.h>

using namespace std;

jack_port_t *_opport;
jack_client_t *_client;
const long int BUFSIZE = 262144;  // max recording space.
float buf[BUFSIZE];
bool done = false;

/* This function created by Fons Adriaensen
 */
int jack_callback (jack_nframes_t nframes, void* /*arg*/)
{
    jack_transport_state_t     state; 
    jack_position_t            posit;
    float                      *p;
    jack_nframes_t             beg, end, k;

    if(done) return 0;

    state = jack_transport_query (_client, &posit);
    p = (float *) jack_port_get_buffer (_opport, nframes);
    if (state == JackTransportRolling)
    {
	end = posit.frame + nframes;
	if( end > BUFSIZE ) end = BUFSIZE;
	for( beg = posit.frame, k=0 ; (beg<end) && (k<nframes) ; ++beg, ++k) {
	    buf[beg] = p[k];
	}
	if( posit.frame > BUFSIZE ) done = true;
    }

    return 0;  
}

int main(void)
{
    jack_client_t *client;

    memset (buf, 0, BUFSIZE * sizeof (float));

    _client = jack_client_open("test_sink",
			      JackNullOption,
			      0);
    _opport = jack_port_register(_client,
				 "out",
				 JACK_DEFAULT_AUDIO_TYPE,
				 JackPortIsInput,
				 0);
    jack_set_process_callback(_client,
			      jack_callback,
			      0);
				 
    jack_activate(_client);

    while(!done) {
	sleep(1);
    }

    jack_nframes_t pos;
    for( pos=0 ; pos<BUFSIZE ; ++pos ) {
	if( buf[pos] > 0.999 ) {
	    cout << pos << endl;
	}
    }
    done = false;

    jack_client_close(_client);

    return 0;
}
