/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   Jack Transport Audit Utils                                            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version                                                               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* t_sink_midi_jack.cpp
 *
 * JACK Transport client that reports the first 16 3-byte MIDI events
 * that it receives, and the frame where they happened.  Note that this
 * includes note on / note off events.
 */

#include <cstring>
#include <unistd.h>
#include <iostream>
#include <iomanip>
#include <jack/jack.h>
#include <jack/transport.h>
#include <jack/midiport.h>

using namespace std;

jack_port_t *_opport;
jack_client_t *_client;
struct mev {
    jack_nframes_t frame;
    jack_midi_data_t buf[3];
};
const long int BUFSIZE = 16;  // max recording space.
mev buf[BUFSIZE];
int bpos = 0;
bool done = false;

int jack_callback (jack_nframes_t nframes, void* /*arg*/)
{
    jack_transport_state_t     state; 
    jack_position_t            posit;
    void*                      port_buf;
    jack_nframes_t             beg, end, k;
    jack_nframes_t             count;
    jack_midi_event_t          ev;

    if(done) return 0;

    state = jack_transport_query (_client, &posit);
    if(state != JackTransportRolling) return 0;

    port_buf = jack_port_get_buffer (_opport, nframes);
    count = jack_midi_get_event_count(port_buf);
    for( k=0 ; k<count ; ++k ) {
	jack_midi_event_get(&ev, port_buf, k);
	if( (bpos < BUFSIZE) && (ev.size == 3) ) {
	    buf[bpos].frame = posit.frame + ev.time;
	    buf[bpos].buf[0] = ev.buffer[0];
	    buf[bpos].buf[1] = ev.buffer[1];
            buf[bpos].buf[2] = ev.buffer[2];
	    ++bpos;
	}
    }
	
    if( bpos >= BUFSIZE ) done = true;

    return 0;  
}

int main(void)
{
    jack_client_t *client;

    memset (buf, 0, BUFSIZE * sizeof (float));

    _client = jack_client_open(__FILE__,
			      JackNullOption,
			      0);
    _opport = jack_port_register(_client,
				 "in",
				 JACK_DEFAULT_MIDI_TYPE,
				 JackPortIsInput,
				 0);
    jack_set_process_callback(_client,
			      jack_callback,
			      0);
				 
    jack_activate(_client);

    while(!done) {
	sleep(1);
    }

    jack_nframes_t pos;
    for( pos=0 ; pos<BUFSIZE ; ++pos ) {
	cout.fill(' ');
	cout << setw(8) << dec << buf[pos].frame << ": ";
	cout.fill('0');
	cout << setw(2) << hex << static_cast<unsigned>(buf[pos].buf[0]) << " "
	     << setw(2) << hex << static_cast<unsigned>(buf[pos].buf[1]) << " "
	     << setw(2) << hex << static_cast<unsigned>(buf[pos].buf[2])
	     << endl;
    }
    done = false;

    jack_client_close(_client);

    return 0;
}
