/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   Jack Transport Audit Utils                                            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version                                                               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* t_source_audio.cpp
 *
 * JACK Transport client that emits silences except for a single, full-scale
 * sample every 22050 frames (when the transport is rolling).
 */

#include <cstring>
#include <unistd.h>
#include <jack/jack.h>
#include <jack/transport.h>

#define FR_INTERVAL (22050)
jack_port_t *_opport;
jack_client_t *_client;

/* This function created by Fons Adriaensen
 */
int jack_callback (jack_nframes_t nframes, void* /*arg*/)
{
    jack_transport_state_t     state; 
    jack_position_t            posit;
    float                      *p;
    unsigned int               k;

    state = jack_transport_query (_client, &posit);
    p = (float *) jack_port_get_buffer (_opport, nframes);
    memset (p, 0, nframes * sizeof (float));
    if (state == JackTransportRolling)
    {
	k = posit.frame % FR_INTERVAL;
	if (k) k = FR_INTERVAL - k;
	if (k < nframes) p [k] = 1.0f;
    }

    return 0;  
}

int main(void)
{
    jack_client_t *client;

    _client = jack_client_open("test_source",
			      JackNullOption,
			      0);
    _opport = jack_port_register(_client,
				 "out",
				 JACK_DEFAULT_AUDIO_TYPE,
				 JackPortIsOutput,
				 0);
    jack_set_process_callback(_client,
			      jack_callback,
			      0);
				 
    jack_activate(_client);

    while(1) {
	sleep(10);
    }

    jack_client_close(_client);

    return 0;
}
