/**********************************************-*- indent-tabs-mode:nil; -*-
 *                                                                         *
 *   Jack Transport Audit Utils                                            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version                                                               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* t_source_midi_jack.cpp
 *
 * JACK Transport client that emits a 1-sample long note on/note off event
 * every 22050 frames (when the transport is rolling).  The note transmitted
 * is a C3.
 */


#include <cstring>
#include <unistd.h>
#include <jack/jack.h>
#include <jack/transport.h>
#include <jack/midiport.h>

#define FR_INTERVAL (22050)
jack_port_t *_opport;
jack_client_t *_client;

/* Outputs a C3 at full velocity every FR_INTERVAL frames.
   Transmits on channel 1.
 */
int jack_callback (jack_nframes_t nframes, void* /*arg*/)
{
    jack_transport_state_t     state; 
    jack_position_t            posit;
    unsigned int               k;
    void*                      port_buf;
    jack_midi_data_t           *buffer;
    static bool                carryover = false;

    state = jack_transport_query(_client, &posit);
    port_buf = jack_port_get_buffer(_opport, nframes);
    jack_midi_clear_buffer(port_buf);
    if (state == JackTransportRolling)
    {
	k = posit.frame % FR_INTERVAL;
	if (k) k = FR_INTERVAL - k;
	if (k < nframes) {
	    if (carryover) {
		carryover = false;
		buffer = jack_midi_event_reserve(port_buf, 0, 3);
		buffer[0] = 0x80; // note off, channel 1
		buffer[1] = 36;   // C3
		buffer[2] = 0x00;
	    }
	    buffer = jack_midi_event_reserve(port_buf, k, 3);
	    buffer[0] = 0x90; // note on, channel 1
	    buffer[1] = 36;   // C3
	    buffer[2] = 0x7F; // full velocity
	    if( k == (nframes-1) ) {
		carryover = true;
	    } else {
		buffer = jack_midi_event_reserve(port_buf, k+1, 3);
		buffer[0] = 0x80; // note off, channel 1
		buffer[1] = 36;   // C3
		buffer[2] = 0x00;
	    }
	}
    }

    return 0;  
}

int main(void)
{
    jack_client_t *client;

    _client = jack_client_open(__FILE__,
			      JackNullOption,
			      0);
    _opport = jack_port_register(_client,
				 "out",
				 JACK_DEFAULT_MIDI_TYPE,
				 JackPortIsOutput,
				 0);
    jack_set_process_callback(_client,
			      jack_callback,
			      0);
				 
    jack_activate(_client);

    while(1) {
	sleep(10);
    }

    jack_client_close(_client);

    return 0;
}
